%%
% AWLSYN                 (fixed-order) Anti-Windup Lyapunov-based SYNthesis
% -------------------------------------------------------------------------
%
% This function  computes a  fixed-order (possibly static) anti-windup com-
% pensator with fixed poles  for a system affected by normalized deadzones.
% The method is based on LMI optimization of  a quadratic Lyapunov function
% and S-procedure relaxation using local sector conditions.   The functions
% are optimized to maximize the ellipsoidal stability region in a given di-
% rection specified via the input argument w or by minimizing the energy of
% the output signal z.
% The anti-windup controller J(s) is computed from a linear interconnection
% M(s) corresponding  to the nominal closed-loop plant.   The global struc- 
% ture is depicted below:
%
%       -----------------------------------------------------
%      |                                   -----             |
%      |                           -------| DZN |<-----      | 
%      |                          |        -----       |     |
%      |                       ws |                    | zs  |
%      |                          |     -----------    |     |
%      |        --------           --->|           |---      |
%      |    v  |        |      w       |    M(s)   |---------     z
%       ------>|  J(s)  |------------->|           |---------------> 
%              |        |               -----------                
%               --------
% 
% The routine may be called in two ways:
%   * by a standard "matlab-only" approach using 
%   * by using the simulink interface SAWLIB
%
% WARNING! The above diagram structure is quite generic. The Lyapunov-based
% approach implemented in this routine is only applicable  if the intercon-
% nection M(s) is such that the v signal verifies either (1) or (2) :
%    (1)  v = T.ws, where T is nonsingular ndz-by-ndz matrix
%    (2)  v = T.ws, where T is a nv-by-ndz matrix with ndz-nv null columns
% The first property must be satisfied in the full-order case.
%
% STANDARD MATLAB CALL 
% --------------------
%  [J,P,crit]=awlsyn(sysM,ndz,nv,x,poles,opt)
%
%  * sysM     state-space description of the linear interconnection M(s) 
%  * ndz      number of deadzone functions
%  * nv       size of the anti-windup output signal v
%  * x        row vector [x1...xr] with r<=n shaping the stability domain.
%             xi is associated to the i-th state space direction ui
%              * xi>0  => constrains P / E(P) contains xi*ui 
%              * xi<0  => maximizes rho / E(P) contains rho*ui
%  * poles    poles of the anti-windup compensator J(s)
%              * poles=[] : static anti-windup (default)
%              * poles=[p1...pr] : fixed poles (do not repeat complex) 
%              * poles='full' : full-order anti-windup
%  * opt      optional structured object with fields: 
%              * solver: LMI solver options (default = [0.01 1000 0 10 1]).
%              * lambda: spectral abscissa constraint in full-order case
%
% CALL SHORTCUT WITH SAWLIB INTERFACE
% -----------------------------------
%  [J,P,crit]=AWLSYN(model,poles,opt)
%  
%  * model    name of  the  Simulink diagram specifying the anti-windup de-
%             sign problem. Use the SAWLIB  Library and run  the short demo 
%             to learn how to generate compatible diagrams.
%  * poles    poles of the anti-windup compensator J(s)
%  * opt      optional structured object with fields: 
%              * solver: LMI solver options (default = [0.01 1000 0 10 1]).
%          
% OUTPUT ARGUMENTS
% ----------------
%  * J        anti-windup compensator
%  * P        positive-definite  matrix defining  the ellipsoidal stability
%             region : E(P)={x/x'Px<=1}.
%  * crit     if x has negative entries:
%               * crit=max(rho) / E(P) contains rho*uj with j in J(w)
%                 In this cas, the output signal zp (if any) is ignored
%             if x has only positive entries:
%               * crit = min(gamma) 
%                  forall x in E(P), ||zp|| < gamma
%                  and where E(P) contains wi*ui
%
% SEE ALSO
% --------
%  * SAWLIB   A specific Lib to generate compatible diagrams
%  * slksaw   Conversion routine
%  * satsan   Saturated systems analysis routine
%  * awhsyn   Anti-Windup design via Hinfinity optimization
%
% -------------------------------------------------------------------------
% SMAC Toolbox ONERA The French Aerospace Lab, Toulouse, France
% SAAW Library for Saturated systems Analysis and Anti-Windup design
% Authors: J-M. Biannic and C. Roos
% Last revision: October 2015 by J-M. Biannic
% http://w3.onera.fr/smac
% -------------------------------------------------------------------------
